@@ -1,6 +1,8 @@
 //! Unsigned (without proofs) transaction
 
 use super::input::{Input, UnsignedInput};
+#[cfg(feature = "json")]
+use super::json;
 use super::prover_result::ProverResult;
 use super::DataInput;
 use super::{
@@ -10,22 +12,28 @@ use super::{
 use ergotree_interpreter::sigma_protocol::prover::ProofBytes;
 #[cfg(feature = "json")]
 use serde::{Deserialize, Serialize};
+#[cfg(feature = "json")]
+use std::convert::TryFrom;
 
 /// Unsigned (inputs without proofs) transaction
 #[cfg_attr(feature = "json", derive(Serialize, Deserialize))]
+#[cfg_attr(
+    feature = "json",
+    serde(
+        try_from = "json::transaction::UnsignedTransactionJson",
+        into = "json::transaction::UnsignedTransactionJson"
+    )
+)]
 #[derive(PartialEq, Debug, Clone)]
 pub struct UnsignedTransaction {
     tx_id: TxId,
     /// unsigned inputs, that will be spent by this transaction.
-    #[cfg_attr(feature = "json", serde(rename = "inputs"))]
     pub inputs: Vec<UnsignedInput>,
     /// inputs, that are not going to be spent by transaction, but will be reachable from inputs
     /// scripts. `dataInputs` scripts will not be executed, thus their scripts costs are not
     /// included in transaction cost and they do not contain spending proofs.
-    #[cfg_attr(feature = "json", serde(rename = "dataInputs"))]
     pub data_inputs: Vec<DataInput>,
     /// box candidates to be created by this transaction
-    #[cfg_attr(feature = "json", serde(rename = "outputs"))]
     pub output_candidates: Vec<ErgoBoxCandidate>,
 }
 
@@ -83,6 +91,30 @@ impl UnsignedTransaction {
     }
 }
 
+#[cfg(feature = "json")]
+impl From<UnsignedTransaction> for json::transaction::UnsignedTransactionJson {
+    fn from(v: UnsignedTransaction) -> Self {
+        json::transaction::UnsignedTransactionJson {
+            inputs: v.inputs,
+            data_inputs: v.data_inputs,
+            outputs: v.output_candidates,
+        }
+    }
+}
+
+#[cfg(feature = "json")]
+impl TryFrom<json::transaction::UnsignedTransactionJson> for UnsignedTransaction {
+    // We never return this type but () fails to compile (can't format) and ! is experimental
+    type Error = String;
+    fn try_from(tx_json: json::transaction::UnsignedTransactionJson) -> Result<Self, Self::Error> {
+        Ok(UnsignedTransaction::new(
+            tx_json.inputs,
+            tx_json.data_inputs,
+            tx_json.outputs,
+        ))
+    }
+}
+
 #[cfg(test)]
 pub mod tests {
     use super::*;
